@@ -7,7 +7,7 @@ RDF::Query::Plan::NamedGraph - Executable query plan for named graphs.
 
 =head1 VERSION
 
-This document describes RDF::Query::Plan::NamedGraph version 2.908_01.
+This document describes RDF::Query::Plan::NamedGraph version 2.909_01.
 
 =head1 METHODS
 
@@ -29,7 +29,7 @@ use base qw(RDF::Query::Plan);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.908_01';
+	$VERSION	= '2.909_01';
 }
 
 ######################################################################
@@ -66,7 +66,7 @@ sub execute ($) {
 	my $l		= Log::Log4perl->get_logger("rdf.query.plan.namedgraph");
 	$l->trace('executing named graph plan');
 	my $model	= $context->model;
-	my $graphs	= $model->get_contexts;
+	my $graphs	= $model->get_graphs;
 	$self->[0]{graphs}	= $graphs;
 	$self->[0]{bound}	= $context->bound || {};
 	$self->[0]{context}	= $context;
